@@ -11,7 +11,9 @@ if sys.version_info < (2, 7):
 def create_venv(name):
     tmpdir = udir.join(name)
     try:
-        subprocess.check_call(['virtualenv', '--distribute',
+        subprocess.check_call(['virtualenv', 
+            #'--never-download', <= could be added, but causes failures
+            # in random cases on random machines
                                '-p', os.path.abspath(sys.executable),
                                str(tmpdir)])
     except OSError as e:
